/* Code for COMP 102 lectures
 * Name:pondy
 */

import ecs100.*;
import java.awt.Color;

/** A Butterfly object represents a butterfly that is drawn in a
window.  The butterfly is shown on the window as two ovals,
with coloured spots.
The class needs a constructor that takes a 
position (x and y) to start the butterfly, and will draw the
new butterfly on the canvas.
The move method takes a change in x and a change in y to move
the position of the butterfly.
The class will also need methods to erase and draw the butterfly.
The move method pick methods should first call the erase method, then
change the relevant field values, and then call the draw method.
 */
public class Butterfly{

    //Fields
    // needs fields to store the DrawingCanvas it is drawn on, along with
    // the position and spot colour of the butterfly.
    private double x;             // location of the center of the center of the butterfly
    private double y;
    private double dx;
    private double dy;
    private Color spotColor;

    private double length = 40;
    private double wing = 5+Math.random()*25;
    private boolean growing = false;
    private double minWing = 8;
    private double maxWing = 30;

    /** Constructor:  Requires the initial position */
    public Butterfly(double x, double y){
        this.x = x;
        this.y = y;
        double dir = Math.random()*Math.PI;
        this.dx = Math.cos(dir);
        this.dy = -Math.sin(dir);
        this.spotColor = Color.getHSBColor((float)Math.random(), 1, 1);
        this.draw();
    }

    /** Move will change the position of the butterfly while flapping
     * its wings a little bit.
     * The argument specifies how big a step to take.
     * It will first erase the butterfly, then change its x and y,
     * and then draw it again.
     */
    public void move(double speed){
        this.erase();
        this.x = this.x + speed*dx;
        this.y = this.y + speed*dy;
        this.bounce();
        this.flap();
        this.draw();
    }

    /** Make the butterfly drop to the bottom of the window and
     * (mostly) close its wings.
     */
    public void land(){
        this.erase();
        this.y = ButterflyGrove.GROUND_LEVEL-length;
        this.wing = minWing;
        this.draw();
        this.growing = true;
    }

    private void flap(){
        if (this.wing <= minWing) this.growing = true;
        if (this.wing >= maxWing) this.growing = false;
        if (this.growing)
            this.wing = this.wing + 5;
        else
            this.wing = this.wing - 5;
    }

    /** Make sure butterfly stays in window and above ground level */
    private void bounce(){
        if (this.x < this.wing){
            this.dx = 1;
            this.x = this.wing;
        }
        if (this.x > 500-this.wing){
            this.dx = - 1;
            this.x = 500-this.wing;
        }
        if (this.y < 0){
            this.dy = 1;
            this.y = 0;
        }
        if (this.y > ButterflyGrove.GROUND_LEVEL-this.length-20){
            this.dy = - 1;
            this.y = ButterflyGrove.GROUND_LEVEL-this.length-20;
        }
    }

    private void draw(){
        double left = this.x-(this.wing)+1;
        double mid = this.x;
        double top = this.y;

        UI.setColor(Color.blue);
        UI.fillOval(left, top, this.wing, this.length);
        UI.fillOval(mid, top, this.wing, this.length);
        UI.setColor(this.spotColor);
        UI.fillOval(mid-(this.wing*2/3), top+(this.length/6), this.wing*2/3, this.length*2/3);
        UI.fillOval(mid, top+(this.length/6), this.wing*2/3, this.length*2/3);
        UI.repaintGraphics();
        UI.sleep(100);
    }

    private void erase(){
        double left = this.x-(this.wing)+1;
        double mid = this.x;
        double top = this.y;

        UI.eraseOval(left-1, top-1, this.wing+2, this.length+2);
        UI.eraseOval(mid-1, top-1, this.wing+2, this.length+2);
    }

}
